@@ -69,7 +69,6 @@ unsigned NAME##_SERIAL::serial = \
 namespace GiNaC {
 
 class function;
-class symmetry;
 
 typedef ex (* eval_funcp)();
 typedef ex (* evalf_funcp)(PyObject* parent);
@@ -77,6 +76,7 @@ typedef ex (* conjugate_funcp)();
 typedef ex (* real_part_funcp)();
 typedef ex (* imag_part_funcp)();
 typedef ex (* derivative_funcp)();
+typedef ex (* expl_derivative_funcp)();
 typedef ex (* power_funcp)();
 typedef ex (* series_funcp)();
 typedef ex (* subs_funcp)(PyObject* parent);
@@ -107,6 +107,9 @@ typedef ex (* derivative_funcp_1)(const ex &, unsigned);
 typedef ex (* derivative_funcp_2)(const ex &, const ex &, unsigned);
 typedef ex (* derivative_funcp_3)(const ex &, const ex &, const ex &, unsigned);
 typedef ex (* derivative_funcp_6)(const ex &, const ex &, const ex &, const ex &, const ex &, const ex &, unsigned);
+typedef ex (* expl_derivative_funcp_1)(const ex &, const symbol &);
+typedef ex (* expl_derivative_funcp_2)(const ex &, const ex &, const symbol &);
+typedef ex (* expl_derivative_funcp_3)(const ex &, const ex &, const ex &, const symbol &);
 typedef ex (* power_funcp_1)(const ex &, const ex &);
 typedef ex (* power_funcp_2)(const ex &, const ex &, const ex &);
 typedef ex (* power_funcp_3)(const ex &, const ex &, const ex &, const ex &);
@@ -127,6 +130,7 @@ typedef ex (* conjugate_funcp_exvector)(const exvector &);
 typedef ex (* real_part_funcp_exvector)(const exvector &);
 typedef ex (* imag_part_funcp_exvector)(const exvector &);
 typedef ex (* derivative_funcp_exvector)(const exvector &, unsigned);
+typedef ex (* expl_derivative_funcp_exvector)(const exvector &, const symbol &);
 typedef ex (* power_funcp_exvector)(const exvector &, const ex &);
 typedef ex (* series_funcp_exvector)(const exvector &, const relational &, int, unsigned);
 typedef void (* print_funcp_exvector)(const exvector &, const print_context &);
@@ -177,6 +181,10 @@ public:
     function_options & derivative_func(derivative_funcp_3 d);
     function_options & derivative_func(derivative_funcp_6 d);
 
+    function_options & expl_derivative_func(expl_derivative_funcp_1 d);
+    function_options & expl_derivative_func(expl_derivative_funcp_2 d);
+    function_options & expl_derivative_func(expl_derivative_funcp_3 d);
+
     function_options & power_func(power_funcp_1 d);
     function_options & power_func(power_funcp_2 d);
     function_options & power_func(power_funcp_3 d);
@@ -239,7 +247,6 @@ public:
 	function_options & remember(unsigned size, unsigned assoc_size=0,
 	                            unsigned strategy=remember_strategies::delete_never);
 	function_options & overloaded(unsigned o);
-	function_options & set_symmetry(const symmetry & s);
 
 	std::string get_name() const { return name; }
 	unsigned get_nparams() const { return nparams; }
@@ -272,12 +279,14 @@ public:
 
 	unsigned nparams;
 
+	eval_funcp pynac_eval_f;
 	eval_funcp eval_f;
 	evalf_funcp evalf_f;
 	conjugate_funcp conjugate_f;
 	real_part_funcp real_part_f;
 	imag_part_funcp imag_part_f;
 	derivative_funcp derivative_f;
+        expl_derivative_funcp expl_derivative_f;
 	power_funcp power_f;
 	series_funcp series_f;
         subs_funcp subs_f;
@@ -301,6 +310,7 @@ public:
 	bool real_part_use_exvector_args;
 	bool imag_part_use_exvector_args;
 	bool derivative_use_exvector_args;
+        bool expl_derivative_use_exvector_args;
 	bool power_use_exvector_args;
 	bool series_use_exvector_args;
 	bool print_use_exvector_args;
@@ -343,6 +353,8 @@ public:
     function(unsigned ser, const ex & param1);
     function(unsigned ser, const ex & param1, const ex & param2);
     function(unsigned ser, const ex & param1, const ex & param2, const ex & param3);
+    function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4);
+    function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4, const ex & param5);
     function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4, const ex & param5, const ex & param6);
 	// end of generated lines
 	function(unsigned ser, exprseq  es);
@@ -360,6 +372,7 @@ public:
 	ex series(const relational & r, int order, unsigned options = 0) const override;
         void useries(flint_series_t& fp, int order) const override;
         ex subs(const exmap & m, unsigned options = 0) const override;
+	ex normal(exmap & repl, exmap & rev_lookup, int level = 0, unsigned options = 0) const override;
 	ex thiscontainer(const exvector & v) const override;
 	ex thiscontainer(std::unique_ptr<exvector> vp) const override;
 	ex conjugate() const override;
@@ -380,6 +393,7 @@ protected:
 	// non-virtual functions in this class
 protected:
 	ex pderivative(unsigned diff_param) const; // partial differentiation
+        ex expl_derivative(const symbol & s) const; // partial differentiation
 	bool lookup_remember_table(ex & result) const;
 	void store_remember_table(ex const & result) const;
 public:
@@ -415,6 +429,11 @@ inline bool is_the_function(const ex & x)
 #define is_ex_the_function(OBJ, FUNCNAME) (GiNaC::is_the_function<FUNCNAME##_SERIAL>(OBJ))
 
 bool has_function(const ex & x);
+bool has_function(const ex& x, const std::string& s);
+bool has_function(const std::vector<std::string>& v);
+bool has_function(const ex& x,
+                const std::vector<std::string>& v,
+                bool all);
 bool has_symbol_or_function(const ex & x);
 
 } // namespace GiNaC
